<?php

namespace Drupal\display_layout\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefinition;
use Drupal\field_ui\Form\EntityViewDisplayEditForm;

/**
 * Overrides the edit form for the EntityViewDisplay entity type.
 */
class EntityViewDisplayLayoutEditForm extends EntityViewDisplayEditForm {

  /**
   * Layout plugin manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManager
   */
  protected $layoutPluginManager;

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $layouts = $this->getLayoutPluginManager()->getDefinitions();
    $layout_options = [];

    /** @var \Drupal\Core\Layout\LayoutDefinition $layout_definition */
    foreach ($layouts as $key => $layout_definition) {
      // Create new layout option group.
      $category = (string) ($layout_definition->getCategory() ?: $this->t('Other'));
      if (!isset($layout_options[$category])) {
        $layout_options[$category] = [];
      }

      $layout_options[$category][$key] = $layout_definition->getLabel();
    }

    $default = $this->entity->getThirdPartySetting('display_layout', 'layout');

    $form['display_layout'] = [
      '#title' => $this->t('Display layout settings'),
      '#type' => 'details',
      '#collapsible' => TRUE,
    ];
    $form['display_layout']['layout'] = [
      '#title' => $this->t('Select a layout'),
      '#type' => 'select',
      '#empty_value' => '_none',
      '#options' => $layout_options,
      '#default_value' => $default,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getRegions() {
    $regions = [];
    $default_message = $this->t('No field is displayed.');
    $layout = $this->entity->getThirdPartySetting('display_layout', 'layout') ?? NULL;
    if (!empty($layout) && $layout != "_none") {
      $layout_definition = $this->getLayoutPluginManager()->getDefinition($layout, FALSE);
      if ($layout_definition instanceof LayoutDefinition) {
        foreach ($layout_definition->getRegions() as $region_id => $region_data) {
          $regions[$region_id] = [
            'title' => $region_data['label'],
            'message' => $default_message,
          ];
        }
      }
    }
    if (empty($regions)) {
      $regions['content'] = [
        'title' => $this->t('Content'),
        'message' => $default_message,
      ];
    }
    $regions['hidden'] = [
      'title' => $this->t('Disabled', [], ['context' => 'Plural']),
      'message' => $this->t('No field is hidden.'),
    ];
    return $regions;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $layout = $form_state->getValue('layout');
    if (!empty($layout) && $layout != "_none") {
      $this->entity->setThirdPartySetting('display_layout', 'layout', $layout);
    }
    else {
      $this->entity->unsetThirdPartySetting('display_layout', 'layout');
    }
  }

  /**
   * Get the layout plugin manager.
   */
  protected function getLayoutPluginManager() {
    if (empty($this->layoutPluginManager)) {
      $this->layoutPluginManager = \Drupal::service('plugin.manager.core.layout');
    }
    return $this->layoutPluginManager;
  }

}
